#include <windows.h>
#include <plugin.hpp>
#include <mp3reg.h>

BOOL RegQueryStyleCode(
     LPCTSTR lpcszRoot,
     LPCTSTR lpcszStyles,
     LPCTSTR lpcszQueryStyle,
     LPBYTE lpbCode )
{
     BOOL bResult = TRUE;
     HKEY hRoot = NULL;
     HKEY hStyles = NULL;
     BYTE lpData[STYLEDIGITS+1];
     DWORD dwData = sizeof( lpData );
     DWORD dwType;

     bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
          HKEY_CURRENT_USER,
          lpcszRoot,
          0,
          KEY_ALL_ACCESS,
          &hRoot )
     );
     if ( bResult )
     {
          bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
               hRoot,
               lpcszStyles,
               0,
               KEY_ALL_ACCESS,
               &hStyles )
          );
     }
     if ( bResult )
     {
          bResult = ( ERROR_SUCCESS == RegQueryValueEx(
               hStyles,
               lpcszQueryStyle,
               0,
               &dwType,
               lpData,
               &dwData )
          );
     }
     if ( bResult )
     {
          *lpbCode = ( BYTE ) atoi( ( LPCTSTR ) lpData );
     }
     RegCloseKey( hRoot );
     RegCloseKey( hStyles );

     return bResult;
}

BOOL RegQueryStyle(
     LPCTSTR lpcszRoot,
     LPCTSTR lpcszStyles,
     LPCTSTR lpcszStyle )
{
     BOOL bResult = TRUE;
     HKEY hRoot = NULL;
     HKEY hStyles = NULL;
     DWORD dwType;
     DWORD dwValue;
     DWORD dwValueSize = sizeof( dwValue );

     bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
          HKEY_CURRENT_USER,
          lpcszRoot,
          0,
          KEY_ALL_ACCESS,
          &hRoot )
     );
     if ( bResult )
     {
          bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
               hRoot,
               lpcszStyles,
               0,
               KEY_ALL_ACCESS,
               &hStyles )
          );
     }
     if ( bResult )
     {
          bResult = ( ERROR_SUCCESS == RegQueryValueEx(
               hStyles,
               lpcszStyle,
               NULL,
               &dwType,
               ( LPBYTE ) &dwValue,
               &dwValueSize )
          );
     }
     RegCloseKey( hRoot );
     RegCloseKey( hStyles );

     return bResult;
}

BOOL RegQueryStyle(
     LPCTSTR lpcszRoot,
     LPCTSTR lpcszStyles,
     BYTE bStyleCode,
     LPTSTR lpszStyle,
     DWORD dwStyle )
{
     BOOL bResult = TRUE;
     LONG lResult = ERROR_SUCCESS;
     HKEY hRoot = NULL;
     HKEY hStyles = NULL;
     BYTE lpData[STYLEDIGITS+1];
     TCHAR lpszValue[REGKEYLEN];
     DWORD dwData;
     DWORD dwValue;
     DWORD dwType;

     bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
          HKEY_CURRENT_USER,
          lpcszRoot,
          0,
          KEY_ALL_ACCESS,
          &hRoot )
     );
     if ( bResult )
     {
          bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
               hRoot,
               lpcszStyles,
               0,
               KEY_ALL_ACCESS,
               &hStyles )
          );
     }
     if ( bResult )
     {
          for ( int idx = 0; lResult == ERROR_SUCCESS; idx++ )
          {
               dwValue = sizeof( lpszValue );
               dwData = sizeof( lpData );
               lResult = RegEnumValue(
                    hStyles,
                    idx,
                    lpszValue,
                    &dwValue,
                    NULL,
                    &dwType,
                    lpData,
                    &dwData
               );
               if ( bStyleCode == ( BYTE ) atoi( ( LPCTSTR ) lpData ) )
               {
                    strncpy( lpszStyle, lpszValue, dwStyle );
                    break;
               }
          }
          bResult = lResult == ERROR_SUCCESS;
     }
     RegCloseKey( hRoot );
     RegCloseKey( hStyles );

     return bResult;
}

VOID RegCopyValuesStylesToEditHistory( HKEY hStyles, HKEY hEdit )
{
     static const TCHAR lpcszLine[] = "Line";
     TCHAR lpszValue[REGKEYLEN];
     BYTE lpData[STYLEDIGITS+1];
     TCHAR lpszEditValueName[sizeof( lpcszLine ) + sizeof( lpData )];
     TCHAR lpszEditValueNumber[3];
     DWORD dwValue;
     DWORD dwData;
     DWORD dwType;
     LONG lResult;

     for ( int idx = 0; idx < EDIT_HISTORY_ITEMS; idx++ )
     {
          dwValue = sizeof( lpszValue );
          dwData = sizeof( lpData );
          lResult = RegEnumValue(
               hStyles,
               idx,
               lpszValue,
               &dwValue,
               NULL,
               &dwType,
               lpData,
               &dwData
          );
          strcpy( lpszEditValueName, lpcszLine );
          itoa( idx, lpszEditValueNumber, 10 );
          strcat( lpszEditValueName, lpszEditValueNumber );
          if ( lResult != ERROR_SUCCESS )
          {
               dwValue = 0;
          }
          RegSetValueEx(
               hEdit,
               lpszEditValueName,
               0,
               REG_SZ,
               ( CONST BYTE* ) lpszValue,
               dwValue
          );
     }
}

BOOL RegCopyStylesToEditHistory(
     LPCTSTR lpcszRoot,
     LPCTSTR lpcszStyles,
     LPCTSTR lpcszEdit )
{
     BOOL bResult = TRUE;
     HKEY hRoot = NULL;
     HKEY hStyles = NULL;
     HKEY hEdit = NULL;
     DWORD __dw;

     bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
          HKEY_CURRENT_USER,
          lpcszRoot,
          0,
          KEY_ALL_ACCESS,
          &hRoot )
     );
     if ( bResult )
     {
          bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
               hRoot,
               lpcszStyles,
               0,
               KEY_ALL_ACCESS,
               &hStyles )
          );
     }
     if ( bResult )
     {
          bResult = ( ERROR_SUCCESS == RegCreateKeyEx(
               hRoot,
               lpcszEdit,
               0,
               NULL,
               0,
               KEY_ALL_ACCESS,
               NULL,
               &hEdit,
               &__dw )
          );
     }
     if ( bResult )
     {
          RegCopyValuesStylesToEditHistory( hStyles, hEdit );
     }
     RegCloseKey( hRoot );
     RegCloseKey( hStyles );
     RegCloseKey( hEdit );

     return bResult;
}

BOOL RegQueryOptions(
     LPCTSTR lpcszRoot,
     LPCTSTR lpcszOptions,
     LPCTSTR lpcszFlag,
     LPDWORD lpdwFlag )
{
     BOOL bResult = TRUE;
     HKEY hRoot = NULL;
     HKEY hOptions = NULL;
     DWORD dwType;
     DWORD dwFlag = sizeof( *lpdwFlag );

     bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
          HKEY_CURRENT_USER,
          lpcszRoot,
          0,
          KEY_ALL_ACCESS,
          &hRoot )
     );
     if ( bResult )
     {
          bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
               hRoot,
               lpcszOptions,
               0,
               KEY_ALL_ACCESS,
               &hOptions )
          );
     }
     if ( bResult )
     {
          bResult = ( ERROR_SUCCESS == RegQueryValueEx(
               hOptions,
               lpcszFlag,
               NULL,
               &dwType,
               ( LPBYTE ) lpdwFlag,
               &dwFlag )
          );
     }

     RegCloseKey( hRoot );
     RegCloseKey( hOptions );

     return bResult;
}

BOOL RegSaveOptions(
     LPCTSTR lpcszRoot,
     LPCTSTR lpcszOptions,
     LPCTSTR lpcszFlag,
     LPDWORD lpdwFlag )
{
     BOOL bResult = TRUE;
     HKEY hRoot = NULL;
     HKEY hOptions = NULL;

     bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
          HKEY_CURRENT_USER,
          lpcszRoot,
          0,
          KEY_ALL_ACCESS,
          &hRoot )
     );
     if ( bResult )
     {
          bResult = ( ERROR_SUCCESS == RegOpenKeyEx(
               hRoot,
               lpcszOptions,
               0,
               KEY_ALL_ACCESS,
               &hOptions )
          );
     }
     if ( bResult )
     {
          bResult = ( ERROR_SUCCESS == RegSetValueEx(
               hOptions,
               lpcszFlag,
               NULL,
               REG_DWORD,
               ( LPBYTE ) lpdwFlag,
               sizeof( *lpdwFlag ) )
          );
     }

     RegCloseKey( hRoot );
     RegCloseKey( hOptions );

     return bResult;
}

